/*****************************************************************************/
/*  dnd.c - drag and drop functions                                          */
/*  Copyright (C) 1998-2000 Brian Masney <masneyb@seul.org>                  */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify     */
/*  it under the terms of the GNU General Public License as published by     */
/*  the Free Software Foundation; either version 2 of the License, or        */
/*  (at your option) any later version.                                      */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful,          */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
/*  GNU General Public License for more details.                             */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License        */
/*  along with this program; if not, write to the Free Software              */
/*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA      */
/*****************************************************************************/

#include "ftp.h"

/* FIXME - drag and drop could be cleaned up a lot */

static int dnd_remote_file 		( char *url, 
					  GList ** transfers,
					  struct ftp_window_data *wdata );

void
listbox_drag (GtkWidget * widget, GdkDragContext * context,
	      GtkSelectionData * selection_data, guint info, guint32 clk_time,
	      gpointer data)
{
  struct ftp_window_data *wdata;
  struct ftp_file_data *tempfle;
  GList *templist, *filelist;
  size_t totlen, oldlen;
  char *tempstr, *str;
  int curpos;
   
  totlen = 0;
  str = NULL;
  wdata = (struct ftp_window_data *) data;
  if (!check_status (_("Drag-N-Drop"), wdata, 0, 1, 1)) 
    return;

  filelist = wdata->hdata->files;
  templist = GTK_CLIST (wdata->listbox)->selection;
  curpos = 0;
  while (templist != NULL)
    {
      templist = get_next_selection (templist, &filelist, &curpos);
      tempfle = (struct ftp_file_data *) filelist->data;

      if (strcmp (tempfle->file, "..") == 0) 
        continue;

      oldlen = totlen;
      if (GFTP_GET_HOSTNAME (wdata->hdata->ftpdata) == NULL)
        {
          tempstr = g_strdup_printf ("%s://%s/%s%c\n", 
                                 GFTP_GET_URL_PREFIX (wdata->hdata->ftpdata),
                                 GFTP_GET_DIRECTORY (wdata->hdata->ftpdata), 
                                 tempfle->file, tempfle->isdir ? '/' : '\0');
        }
      else if (GFTP_GET_USERNAME (wdata->hdata->ftpdata) == NULL 
          || GFTP_GET_PASSWORD (wdata->hdata->ftpdata) == NULL)
        {
          tempstr = g_strdup_printf ("%s://%s:%d%s/%s%c\n", 
                                 GFTP_GET_URL_PREFIX (wdata->hdata->ftpdata),
                                 GFTP_GET_HOSTNAME (wdata->hdata->ftpdata),
                                 GFTP_GET_PORT (wdata->hdata->ftpdata),
                                 GFTP_GET_DIRECTORY (wdata->hdata->ftpdata), 
                                 tempfle->file, tempfle->isdir ? '/' : '\0');
        }
      else
        {
          tempstr = g_strdup_printf ("%s://%s@%s:%d%s/%s%c\n", 
                                 GFTP_GET_URL_PREFIX (wdata->hdata->ftpdata),
                                 GFTP_GET_USERNAME (wdata->hdata->ftpdata), 
                                 GFTP_GET_HOSTNAME (wdata->hdata->ftpdata),
                                 GFTP_GET_PORT (wdata->hdata->ftpdata),
                                 GFTP_GET_DIRECTORY (wdata->hdata->ftpdata), 
                                 tempfle->file, tempfle->isdir ? '/' : '\0');
        }
      remove_double_slashes (tempstr + 7);
      totlen += strlen (tempstr);
      str = g_realloc (str, totlen + 1);
      strcpy (str + oldlen, tempstr);
      g_free (tempstr);
    }

  if (str != NULL)
    {
      gtk_selection_data_set (selection_data, selection_data->target, 8,
      	                      (unsigned char *) str, strlen (str));
      g_free (str);
    }
}


void
listbox_get_drag_data (GtkWidget * widget, GdkDragContext * context, gint x,
		       gint y, GtkSelectionData * selection_data, guint info,
		       guint32 clk_time, struct ftp_window_data *wdata)
{
  struct ftp_transfer_data *tdata;
  GList *new_file_transfers, *templist;
  char *newpos, *oldpos, tempchar;
  int finish_drag;
   
  if (!check_status (_("Drag-N-Drop"), wdata, 0, 0, 1)) 
    return;

  new_file_transfers = NULL;
  finish_drag = 0;
  if ((selection_data->length >= 0) && (selection_data->format == 8)) 
    {
      oldpos = (char *) selection_data->data;
      while ((newpos = strchr (oldpos, '\n')) 
             || (newpos = strchr (oldpos, '\0'))) 
        {
          tempchar = *newpos;
          *newpos = '\0';
          ftp_log (gftp_logging_misc, NULL, "Received URL %s\n", oldpos);
          if (dnd_remote_file (oldpos, &new_file_transfers, wdata))
            finish_drag = 1;
         
          if (*newpos == '\0') 
            break;
          oldpos = newpos + 1;
        }
    }
  done_with_dirs ();

  if (finish_drag) 
    {
      templist = new_file_transfers;
      while (templist != NULL) 
        {
          tdata = templist->data;
	  if (tdata->fromhdata->files != NULL)
            dotrans (tdata);
          templist = templist->next;
        }
    }
  gtk_drag_finish (context, finish_drag, FALSE, clk_time);
}


static int
dnd_remote_file (char *url, GList ** transfers, struct ftp_window_data *wdata)
{
  struct ftp_transfer_data *tdata;
  gftp_request *current_ftpdata;
  struct ftp_file_data *newfle;
  char *str, *pos;
  GList *templist;
  int i;

  newfle = g_malloc0 (sizeof (struct ftp_file_data));
  newfle->shown = 1;
  if (url[strlen (url) - 1] == '/') 
    {
      newfle->isdir = 1;
      url[strlen (url) - 1] = '\0';
    }

  current_ftpdata = gftp_request_new ();
  gftp_set_proxy_config (current_ftpdata, proxy_config);
  current_ftpdata->proxy_hosts = proxy_hosts;

  if (gftp_init_url (current_ftpdata, url) != 0) 
    {
      ftp_log (gftp_logging_misc, NULL, 
               _("Drag-N-Drop: Ignoring url %s: Not a valid url\n"), url);
      gftp_request_destroy (current_ftpdata);
      free_fdata (newfle);
      return (0);
    }

  if (strcmp (GFTP_GET_USERNAME (current_ftpdata), ANON_LOGIN) == 0) 
    gftp_set_password (current_ftpdata, emailaddr);

  if ((str = GFTP_GET_DIRECTORY (current_ftpdata)) != NULL) 
    {
      if ((pos = strrchr (str, '/')) == NULL) 
        pos = str;
      else pos++;
      *(pos - 1) = '\0';
      i = 1;
    }
  else 
    {
      pos = str = "";
      i = 0;
    }

  if (compare_hdata_structs (current_ftpdata, wdata->hdata->ftpdata, 1))
    return (0);

  if (i)
    *(pos - 1) = '/';

  newfle->file = g_malloc (strlen (str) + 1);
  strcpy (newfle->file, str);
  newfle->remote_file = g_strconcat (GFTP_GET_DIRECTORY (wdata->hdata->ftpdata),
                                     "/", pos, NULL);
  newfle->ascii = get_file_transfer_mode (newfle->file);
  gftp_set_directory (current_ftpdata, "");

  tdata = NULL;
  templist = *transfers;
  while (templist != NULL) 
    {
      tdata = (struct ftp_transfer_data *) templist->data;
      if (compare_hdata_structs (tdata->fromhdata->ftpdata, current_ftpdata, 1))
        break;
      templist = templist->next;
    }

  if (tdata == NULL) 
    {
      tdata = new_tdata ();
      tdata->direction = 1;
      gftp_request_destroy (tdata->fromhdata->ftpdata);
      tdata->fromhdata->ftpdata = current_ftpdata;
      if ((GFTP_GET_PASSWORD (tdata->fromhdata->ftpdata) == NULL 
          || *GFTP_GET_PASSWORD (tdata->fromhdata->ftpdata) == '\0') 
          && compare_hdata_structs (tdata->fromhdata->ftpdata, 
          wdata->hdata->ftpdata, 0))
        gftp_set_password (tdata->fromhdata->ftpdata, 
                           GFTP_GET_PASSWORD (wdata->hdata->ftpdata));
      copy_hdata_struct (wdata->hdata, tdata->tohdata);
      wdata->hdata->ftpdata->init (tdata->tohdata->ftpdata);
      *transfers = g_list_append (*transfers, tdata);
    }
  else
    gftp_request_destroy (current_ftpdata);

  if (newfle->isdir)
    {
      add_entire_directory (tdata, newfle, 
		            GFTP_GET_DIRECTORY (tdata->fromhdata->ftpdata), 
			    GFTP_GET_DIRECTORY (tdata->tohdata->ftpdata), 
			    tdata->fromhdata->ftpdata);
    }
  else
    {
      tdata->fromhdata->files = g_list_append (tdata->fromhdata->files, newfle);
      tdata->fromhdata->totalfiles++;
      if (tdata->curfle == NULL) 
	tdata->curfle = tdata->fromhdata->files;
    }
  return (1);
}


void
openurl_get_drag_data (GtkWidget * widget, GdkDragContext * context, gint x,
		       gint y, GtkSelectionData * selection_data, guint info,
		       guint32 clk_time, struct ftp_window_data *wdata)
{
  void (*init)(gftp_request * request);
  char *pos;
  int i;

  if ((selection_data->length >= 0) && (selection_data->format == 8)) 
    {
      if ((pos = strstr (selection_data->data, "://")) == NULL)
        {
          ftp_log (gftp_logging_error, NULL,
                   _("Drag-N-Drop: Ignoring url %s: Not a valid url\n"),
                   selection_data->data);
          return;
        }

      *pos = '\0';
      for (i = 0; gftp_protocols[i].url_prefix; i++)
        {
          if (strcmp (gftp_protocols[i].url_prefix, selection_data->data) == 0)
            break;
        }

      if (gftp_protocols[i].url_prefix == NULL)
        {
          ftp_log (gftp_logging_error, NULL,
                   _("Error: The protocol %s is currently not supported\n"),
                   selection_data->data);
          return;
        }
      *pos = ':';

      init = current_wdata->hdata->ftpdata->init;
      gftp_protocols[i].init (current_wdata->hdata->ftpdata);
      if (gftp_parse_url (current_wdata->hdata->ftpdata, 
                          (char *) selection_data->data) == 0) 
        {
          if (strcmp (GFTP_GET_USERNAME (current_wdata->hdata->ftpdata), 
                      ANON_LOGIN) == 0)
            gftp_set_password (current_wdata->hdata->ftpdata, emailaddr);
          if (IS_CONNECTED (current_wdata))
            disconnect (current_wdata);
          ftp_connect (current_wdata->hdata, 1);
        }
      else 
        {
          ftp_log (gftp_logging_misc, NULL, 
                   _("Could not parse URL %s\n"), selection_data->data);
          init (current_wdata->hdata->ftpdata);
        }
    }
}

